﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization.Compiler;
using Microsoft.Xna.Framework.Graphics;

namespace PolygonHead
{
    public struct Character
    {
        public CharacterVertex VertexA;

        public CharacterVertex VertexB;

        public CharacterVertex VertexC;

        public CharacterVertex VertexD;

        public Vector4 Color
        {
            get
            {
                return this.VertexA.Color;
            }
            set
            {
                this.VertexA.Color = value;
                this.VertexB.Color = value;
                this.VertexC.Color = value;
                this.VertexD.Color = value;
            }
        }

        public void ApplyOffset(ref Vector2 iOffset)
        {
            this.VertexA.Position.X = this.VertexA.Position.X + iOffset.X;
            this.VertexA.Position.Y = this.VertexA.Position.Y + iOffset.Y;
            this.VertexB.Position.X = this.VertexB.Position.X + iOffset.X;
            this.VertexB.Position.Y = this.VertexB.Position.Y + iOffset.Y;
            this.VertexC.Position.X = this.VertexC.Position.X + iOffset.X;
            this.VertexC.Position.Y = this.VertexC.Position.Y + iOffset.Y;
            this.VertexD.Position.X = this.VertexD.Position.X + iOffset.X;
            this.VertexD.Position.Y = this.VertexD.Position.Y + iOffset.Y;
        }

        public static Character FromGlyph(Glyph iGlyph, Vector4 iColor, int iTextureWidth, float iTextureHeight)
        {
            Character leftSideBearing = new Character();
            if (iGlyph.ForceWhite)
            {
                //	float single1.Y = float single2 = single1.X = single2;
            }
            Vector2 x = new Vector2();
            x.X = (float)iGlyph.Origin.X / (float)iTextureWidth;
            x.Y = (float)iGlyph.Origin.Y / (float)iTextureHeight;
            Vector2 y = new Vector2();
            y.X = x.X + (float)iGlyph.Size.X / (float)iTextureWidth;
            y.Y = x.Y + (float)iGlyph.Size.Y / (float)iTextureHeight;
            leftSideBearing.VertexA.Position.X = (float)iGlyph.LeftSideBearing;
            leftSideBearing.VertexA.Position.Y = 0;
            leftSideBearing.VertexA.TexCoord.X = x.X;
            leftSideBearing.VertexA.TexCoord.Y = x.Y;
            leftSideBearing.VertexA.Color = iColor;
            leftSideBearing.VertexB.Position.X = (float)iGlyph.LeftSideBearing + iGlyph.Size.X;
            leftSideBearing.VertexB.Position.Y = 0;
            leftSideBearing.VertexB.TexCoord.X = y.X;
            leftSideBearing.VertexB.TexCoord.Y = x.Y;
            leftSideBearing.VertexB.Color = iColor;
            leftSideBearing.VertexC.Position.X = (float)iGlyph.LeftSideBearing + iGlyph.Size.X;
            leftSideBearing.VertexC.Position.Y = (float)iGlyph.Size.Y;
            leftSideBearing.VertexC.TexCoord.X = y.X;
            leftSideBearing.VertexC.TexCoord.Y = y.Y;
            leftSideBearing.VertexC.Color = iColor;
            leftSideBearing.VertexD.Position.X = (float)iGlyph.LeftSideBearing;
            leftSideBearing.VertexD.Position.Y = (float)iGlyph.Size.Y;
            leftSideBearing.VertexD.TexCoord.X = x.X;
            leftSideBearing.VertexD.TexCoord.Y = y.Y;
            leftSideBearing.VertexD.Color = iColor;
            return leftSideBearing;
        }
    }

    public struct CharacterVertex
    {
        public Vector4 Color;

        public Vector2 Position;

        public const int SIZEINBYTES = 32;

        public Vector2 TexCoord;

        public readonly static VertexElement[] VertexElements;

        static CharacterVertex()
        {
            VertexElement[] vertexElement = new VertexElement[3];
            vertexElement[0] = new VertexElement();//0, 0, 1, 0, 0, 0);
            vertexElement[1] = new VertexElement();//0, 8, 1, 0, 5, 0);
            vertexElement[2] = new VertexElement();//0, 16, 3, 0, 10, 0);
            CharacterVertex.VertexElements = vertexElement;
        }
    }

    public struct Glyph
    {
        public int AdvanceWidth;

        public char Character;

        public bool ForceWhite;

        public int LeftSideBearing;

        public Point Origin;

        public Point Size;

        public static Glyph Read(ContentReader iInput)
        {
            Glyph glyph = new Glyph();
            glyph.Character = iInput.ReadChar();
            glyph.Origin = new Point();
            glyph.Origin.X = iInput.ReadInt32();
            glyph.Origin.Y = iInput.ReadInt32();
            glyph.Size = new Point();
            glyph.Size.X = iInput.ReadInt32();
            glyph.Size.Y = iInput.ReadInt32();
            glyph.AdvanceWidth = iInput.ReadInt32();
            glyph.LeftSideBearing = iInput.ReadInt32();
            glyph.ForceWhite = iInput.ReadBoolean();
            return glyph;
        }

        // my function
        public void Write(ref ContentWriter iOutput)
        {
            iOutput.Write((Char)this.Character);
            iOutput.Write((Int32)this.Origin.X);
            iOutput.Write((Int32)this.Origin.Y);
            iOutput.Write((Int32)this.Size.X);
            iOutput.Write((Int32)this.Size.Y);
            iOutput.Write((Int32)this.AdvanceWidth);
            iOutput.Write((Int32)this.LeftSideBearing);
            iOutput.Write((Boolean)this.ForceWhite);
        }
    }
}