﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;

namespace FontRasterizer.Batch
{
	static class BatchProcess
	{
		public static FontStyle ToFontStyle(this String str)
		{
			FontStyle result = FontStyle.Regular;
			str = str.ToUpper();
			result |= str.IndexOf("B") != -1 ? FontStyle.Bold : FontStyle.Regular;
			result |= str.IndexOf("I") != -1 ? FontStyle.Italic : FontStyle.Regular;
			result |= str.IndexOf("S") != -1 ? FontStyle.Strikeout : FontStyle.Regular;
			result |= str.IndexOf("U") != -1 ? FontStyle.Underline : FontStyle.Regular;

			return result;

		}

		public static void Batch(String SettingFilename)
		{
			String SettingFile = File.ReadAllText(SettingFilename);
			foreach(var line in SettingFile.Split(new []{'\n','\r'},StringSplitOptions.RemoveEmptyEntries))
			{
				//try
				{
					//	MagickaFontName	FontName	FontSize	FontStyle	SeedFilename
					var cell = line.Split(new[] { '\t', '	' }, StringSplitOptions.RemoveEmptyEntries);
					if (cell.Length < 5) continue;

					FontRasterizer rizer = new FontRasterizer();
					rizer.OutputDirectory = cell[0];
					rizer.FontFamilyName = cell[1];
					rizer.FontSize = Double.Parse(cell[2]);
					rizer.FontStyle = cell[3].ToFontStyle();
					rizer.TargetStrings = File.ReadAllText(cell[4]);
					rizer.Prefix = rizer.FontFamilyName + "_";

					Console.WriteLine(rizer.OutputDirectory + "を処理中");
					rizer.Rasterize();
					Console.WriteLine(" フォントサイズ:{0}", rizer.FontSize);
				}
				try { }
				catch (Exception e)
				{
					Console.WriteLine("エラーが発生しました");
					ShowError(e);
					Console.WriteLine("続行しますか？[yN]");
					if (Char.ToUpper(Console.ReadKey(true).KeyChar) != 'Y')
						return;
				}
			}
		}

		private static void ShowError(Exception e)
		{

			Console.WriteLine(e.Message);
			Console.WriteLine("[場所]" + e.TargetSite.ToString());
			Console.WriteLine("[Sorc]" + e.Source);
			if (e.InnerException != null)
			{
				Console.WriteLine(" ==> ");
				ShowError(e.InnerException);
			}
			else
			{
				Console.WriteLine("------");
			}
		}
	}
}
