﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FontRasterizer
{
    public partial class Form1 : Form
    {
        FontRasterizer currentProject = new FontRasterizer();
        Boolean IsProjectLoading = false;
		String advancedFontSeedFile = "uniqueChars.txt";

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
    
            // フォントファミリーの列挙
            foreach (var f in FontFamily.Families)
            {
                if (f.IsStyleAvailable(FontStyle.Regular))
                    comboBox1.Items.Add(f.Name);
            }

            // サブフォルダの列挙
            foreach (var f in (new System.IO.DirectoryInfo(System.IO.Directory.GetCurrentDirectory())).GetDirectories())
            {
                comboBox2.Items.Add(f.Name);
            }

            comboBox3.SelectedIndex = 0;

			// uniqueChars.txtが有れば生成フォントをそこから読み取る

			if (System.IO.File.Exists(advancedFontSeedFile))
			{
				textBox1.Text = System.IO.File.ReadAllText(advancedFontSeedFile);
			}
        }

        #region ユーザ入力をcurrentProjectに格納するまどろっこしい処理
        private void button2_Click(object sender, EventArgs e)
        {
            if (IsProjectLoading) return;
            if (colorDialog1.ShowDialog() == DialogResult.OK)
            {
                currentProject.ForeColor = colorDialog1.Color;
                UpdatePreview();

            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (IsProjectLoading) return;
            try
            {
                currentProject.FontFamilyName = new FontFamily(comboBox1.Text).Name;
                textBox3.Text = comboBox1.Text;
                UpdatePreview();
                
            }
            catch (System.ArgumentException ex)
            {
                System.Media.SystemSounds.Beep.Play();
				Console.WriteLine("エラー:{0}",ex.Message);
            }
            
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            if (IsProjectLoading) return;
            currentProject.TargetStrings = textBox1.Text;
            UpdatePreview();
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            if (IsProjectLoading) return;
            currentProject.FontSize = Math.Max(1,(int)numericUpDown1.Value);
            UpdatePreview();
        }

        private void textBox2_TextChanged(object sender, EventArgs e)
        {
            if (IsProjectLoading) return;
            currentProject.OutputDirectory = comboBox2.Text;
            UpdatePreview();
        }

        private void textBox3_TextChanged(object sender, EventArgs e)
        {
            if (IsProjectLoading) return;
            currentProject.Prefix = textBox3.Text;
            UpdatePreview();
        }

        private void stylebox_CheckedChanged(object sender, EventArgs e)
        {
            var me = sender as CheckBox;
            FontStyle fontstyle = new FontStyle[] { FontStyle.Bold, FontStyle.Italic, FontStyle.Strikeout, FontStyle.Underline }[int.Parse((String)me.Tag)];


            if (me.Checked)
            {
                currentProject.FontStyle = currentProject.FontStyle | fontstyle;
            }
            else
            {
                currentProject.FontStyle = currentProject.FontStyle & ~fontstyle;
            }
            UpdatePreview();
        }

        private void comboBox3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (IsProjectLoading) return;
            currentProject.Method = comboBox3.SelectedIndex;
        }

        private void UpdatePreview()
        {
            textBox1.Font = currentProject.GetFont();
            textBox1.ForeColor = currentProject.ForeColor;
            textBox1.BackColor = currentProject.ForeColor.GetBrightness() < 0.5 ? Color.White : Color.Black;
            button2.BackColor = currentProject.ForeColor;

        }

        // 
        /// <summary>プロジェクトの内容をUIに反映させる。英語は適当
        /// 
        /// </summary>
        private void WritebackProject()
        {
            IsProjectLoading = true;
            try
            {
                textBox3.Text = currentProject.Prefix;
                comboBox1.Text = currentProject.FontFamilyName;
                comboBox2.Text = currentProject.OutputDirectory;
                numericUpDown1.Value = (Decimal)(int)currentProject.FontSize;
                textBox1.Text = currentProject.TargetStrings;
                button2.BackColor = currentProject.ForeColor;
                checkBox1.Checked = (FontStyle.Bold & currentProject.FontStyle) != 0;
                checkBox2.Checked = (FontStyle.Italic & currentProject.FontStyle) != 0;
                checkBox3.Checked = (FontStyle.Strikeout & currentProject.FontStyle) != 0;
                checkBox4.Checked = (FontStyle.Underline & currentProject.FontStyle) != 0;
                comboBox3.SelectedIndex = currentProject.Method;
            }
            finally
            {
                IsProjectLoading = false;
            }
        }

        /// <summary> プロジェクトにUIのを反映させる。英語は適当
        /// 
        /// </summary>
        private void WriteProject()
        {
            currentProject.Prefix = textBox3.Text;
            currentProject.OutputDirectory = comboBox2.Text;
            currentProject.FontSize = (int)numericUpDown1.Value;
            currentProject.TargetStrings = textBox1.Text;
            currentProject.ForeColor = button2.BackColor;
            currentProject.FontStyle = FontStyle.Regular;
            currentProject.FontStyle |= checkBox1.Checked ? FontStyle.Bold : FontStyle.Regular;
            currentProject.FontStyle |= checkBox2.Checked ? FontStyle.Italic : FontStyle.Regular;
            currentProject.FontStyle |= checkBox3.Checked ? FontStyle.Strikeout : FontStyle.Regular;
            currentProject.FontStyle |= checkBox4.Checked ? FontStyle.Underline : FontStyle.Regular;
            currentProject.Method = comboBox3.SelectedIndex;
        }
        #endregion


        private void button1_Click(object sender, EventArgs e)
        {
			Console.WriteLine("処理中.... ");

            WriteProject();

            currentProject.Rasterize();
			Console.WriteLine("フォントサイズ: {0}", currentProject.FontSize);
			Console.WriteLine("処理完了! ");
       }

        private void button3_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            currentProject = FontRasterizer.LoadAsProject(openFileDialog1.FileName);
            System.IO.Directory.SetCurrentDirectory(
                System.IO.Path.GetDirectoryName(openFileDialog1.FileName));

            WritebackProject();
            UpdatePreview();
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            WriteProject();
            currentProject.SaveAsProject(saveFileDialog1.FileName);
            System.IO.Directory.SetCurrentDirectory(
                System.IO.Path.GetDirectoryName(saveFileDialog1.FileName));
        }

        private void label4_Click(object sender, EventArgs e)
        {

        }

		private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
		{

		}

		#region 自動フォントサイズ調整


	

		#endregion


	}

    
}
