﻿using System;
using System.Data;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using Ionic.Zip;

namespace ODSReader
{
	static class ODSReader
	{
		public static DataSet OpenOSDAsDataSet(String filename)
		{
			using (MemoryStream ms = new MemoryStream())
			{
				using (ZipFile zip = ZipFile.Read(filename))
				{
					zip.CaseSensitiveRetrieval = true; // 逆?
					zip["content.xml"].Extract(ms);
				}
				ms.Seek(0, SeekOrigin.Begin);
				return OpenRawOSDAsDataSet(ms);
			}
		}
		public static DataSet OpenOSDAsDataSet(Stream strm)
		{
			using (MemoryStream ms = new MemoryStream())
			{
				using (ZipFile zip = ZipFile.Read(strm))
				{
					zip.CaseSensitiveRetrieval = true; // 逆?
					zip["content.xml"].Extract(ms);
				}
				ms.Seek(0, SeekOrigin.Begin);
				return OpenRawOSDAsDataSet(ms);
			}
		}

		/// <summary>linq使った超簡易content.xmlリーダ。</summary>
		/// <param name="strm"></param>
		/// <returns></returns>
		private static DataSet OpenRawOSDAsDataSet(Stream strm)
		{
			var ds = new DataSet();
			var doc = XDocument.Load(strm);
			var xtables = doc.Descendants().Where(e => e.Name.LocalName == "table");
			foreach (var xtable in xtables)
			{
				var dtable = ds.Tables.Add();
				dtable.TableName = xtable.Attributes().First(e => (e.Name.LocalName == "name")).Value;
				var xrows = xtable.Elements().Where(e => (e.Name.LocalName == "table-row"));
				int columnCount = xrows.Max(e => ( e.Elements().Count()));
				for (int i = 0; i < columnCount; i++)
					dtable.Columns.Add();
				foreach (var xr in xrows)
				{
					var dr = dtable.NewRow();
					int i = 0 ;
					foreach (var xc in xr.Elements().Where(e => e.Name.LocalName == "table-cell"))
					{
						String value = "";
						foreach (var para in xc.Elements().Where(e => e.Name.LocalName == "p"))
							value += para.Value + "\n";
						dr[i] = value;
						i += 1;
					}
					dtable.Rows.Add(dr);
				}
			}

			return ds;
		}

	}
}
